/*
 *	Ohio Trollius
 *	Copyright 1995 The Ohio State University
 *	RBD
 *
 *	$Log:	rev.c,v $
 * Revision 6.1  96/11/23  19:58:16  nevin
 * Ohio Release
 * 
 * Revision 6.0  96/02/29  13:48:49  gdburns
 * Ohio Release
 * 
 * Revision 5.2.1.1  95/11/20  19:03:42  gdburns
 * Cast arguments to rev4() to (void *).
 * 
 * Revision 5.2  94/08/22  13:57:08  gdburns
 * Ohio Release
 * 
 * Revision 5.1  94/05/18  10:57:03  gdburns
 * Ohio Release
 * 
 * Revision 2.3  94/04/22  12:35:05  gdburns
 * Ohio Release
 * 
 *	Function:	- reverse byte ordering of data types
 *			- handles 4 and 8-byte types in fast manner
 *			- handles all other types as byte arrays
 */

#include <portable.h>

/*
 *	rev4
 *
 *	Function:	- reverse byte ordering of a 4-byte array
 *			- array is treated as an unsigned 4-byte integer
 *	Accepts:	- ptr source array
 *			- ptr destination array
 */
void
rev4(src, dest)

void			*src;
void			*dest;

{
	uint4		old;
	uint4		*d;

	old = *((uint4 *) src);
	d = (uint4 *) dest;

	*d = (old & 0x000000FF) << 16;
	*d = (*d | (old & 0x0000FF00)) << 8;
	old >>= 8;
	*d |= (old & 0x0000FF00);
	old >>= 16;
	*d |= (old & 0x000000FF);
}

/*
 *	rev8
 *
 *	Function:	- reverse byte ordering of a 8-byte array
 *			- array is treated as 2 unsigned 4-byte integers
 *	Accepts:	- ptr source array
 *			- ptr destination array
 */
void
rev8(src, dest)

void			*src;
void			*dest;

{
	uint4		old;
	char		*d;

	old = *((uint4 *) src + 1);
	d = (char *) dest + 4;

	rev4(src, (void *) d);
	rev4((void *) &old, dest);
}

/*
 *	revn
 *
 *	Function:	- reverse byte ordering of a multi-byte array
 *	Accepts:	- ptr source array
 *			- ptr destination array
 *			- # of bytes
 */
void
revn(src, dest, nbytes)

void			*src;
void			*dest;
int			nbytes;

{
	char		temp;
	char		*s, *d;

	s = (char *) src;
	d = (char *) dest + nbytes - 1;

	if (src == dest) {
/*
 * Same array, swap bytes from both ends.
 */
		for (nbytes /= 2; nbytes > 0; --nbytes, ++s, --d) {
			temp = *s; *s = *d; *d = temp;
		}
	}
	else {
/*
 * Different arrays, just copy the reversed order.
 * This assumes the arrays do not overlap.
 */
		for ( ; nbytes > 0; --nbytes, ++s, --d) {
			*d = *s;
		}
	}
}

/*
 *	mrev4
 *
 *	Function:	- reverse byte ordering of an array of 4-byte elements
 *	Accepts:	- ptr array
 *			- # elements in array
 */
void
mrev4(array, num)

void			*array;
int			num;

{
	char		*p;

	p = (char *) array;

	for ( ; num > 0; --num, p += 4) {
		rev4((void *) p, (void *) p);
	}
}

/*
 *	mrev8
 *
 *	Function:	- reverse byte ordering of an array of 8-byte elements
 *	Accepts:	- ptr array
 *			- # elements in array
 */
void
mrev8(array, num)

void			*array;
int			num;

{
	char		*p;

	p = (char *) array;
	for ( ; num > 0; --num, p += 8) {
		rev8((void *) p, (void *) p);
	}
}

/*
 *	mrevn
 *
 *	Function:	- reverse byte ordering of an array of n-byte elements
 *	Accepts:	- ptr array
 *			- element size
 *			- # elements in array
 */
void
mrevn(array, size, num)

void			*array;
int			size;
int			num;

{
	char		*p;

	p = (char *) array;
	for ( ; num > 0; --num, p += size) {
		revn((void *) p, (void *) p, size);
	}
}

/*
 *	rw
 *
 *	Function:	- reverse word byte order (old Trollius call)
 *	Accepts:	- original word
 *	Returns:	- word with reversed byte order
 */
uint4
rw(oword)

uint4			oword;

{
	uint4		rword;

	rev4((void *) &oword, (void *) &rword);
	return(rword);
}
